#include <string.h>
#include <fcntl.h>
#include <math.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_mixer.h>
#include <SDL_ttf.h>

#include "sdl_gp2x.h"

SDL_Surface *InitSDL(int w, int h, int d)
{
	SDL_Surface *surface;

	// Initializating SDL
	printf("Initializating SDL...");
	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) < 0) printf("\t\t\t\tERROR\n");
	printf("\t\t\t\tOK\n");

	// Set video mode
	printf("Setting video mode...");
	surface = SDL_SetVideoMode(w, h, d, SDL_DOUBLEBUF | SDL_HWSURFACE);
	if(!surface)
	{
		printf("\t\t\t\tERROR\n");
		SDL_Quit();
	}
	else
		printf("\t\t\t\tOK\n");

	// Disabling SDL cursor
	SDL_ShowCursor(SDL_DISABLE);

	// Return screen surface
	return surface;
}

void QuitSDL()
{
	// Quit SDL
	SDL_Quit();
}

void JoystickOpen(int device)
{
	// Opening joystick device
	printf("Opening joystick device...");
	if(SDL_JoystickOpen(device) < 0)
	{
		printf("\t\t\tERROR\n");
		SDL_Quit();
	}
	else
		printf("\t\t\tOK\n");
}

void DrawPixel(SDL_Surface *dst, int x, int y, Uint32 c)
{
	if ((x >= 320) || (x < 0)) return;
	if ((y >= 240) || (y < 0)) return;

	// Lock surface
	if ( SDL_MUSTLOCK(dst) )
	{
		if ( SDL_LockSurface(dst) < 0 ) return;
	}

	// Surface format
	switch(dst->format->BytesPerPixel)
	{
		// 8-bpp mode
		case 1:
		{
			Uint8 *bufp;

			bufp = (Uint8 *)dst->pixels + y*dst->pitch + x;
			*bufp = c;
        	}
        	break;

		// 16-bpp mode
		case 2:
		{
			Uint16 *bufp;

			bufp = (Uint16 *)dst->pixels + y*dst->pitch/2 + x;
			*bufp = c;
		}
		break;

		// 24-bpp mode
		case 3:
		{
			Uint8 *bufp;

			bufp = (Uint8 *)dst->pixels + y*dst->pitch + x * 3;
			if(SDL_BYTEORDER == SDL_LIL_ENDIAN)
			{
				bufp[0] = c;
				bufp[1] = c >> 8;
				bufp[2] = c >> 16;
			}
			else
			{
				bufp[2] = c;
				bufp[1] = c >> 8;
				bufp[0] = c >> 16;
			}
		}
		break;

		// 32-bpp mode
		case 4:
		{
			Uint32 *bufp;

			bufp = (Uint32 *)dst->pixels + y*dst->pitch/4 + x;
			*bufp = c;
		}
		break;
	}

	// Unlock surface
	if ( SDL_MUSTLOCK(dst) )
	{
		SDL_UnlockSurface(dst);
	}

	// Update surface
	SDL_UpdateRect(dst, x, y, 1, 1);
}

Uint32 GetPixelColor(SDL_Surface *src, int x, int y)
{
	Uint32 color = 0;
	char *pPosition = (char*) src->pixels;

	// Set pixel position
	pPosition += (src->pitch * y);
	pPosition += (src->format->BytesPerPixel * x);

	// Get color
	memcpy(&color, pPosition, src->format->BytesPerPixel);

	// Return color
	return color;
}

void SetGamma(float r, float g, float b)
{
	// Set gamma
	if(SDL_SetGamma(r, g, b) < 0) printf("Error setting gamma.\n");
}

void ShowMouse(int i)
{
	// Active or disable mouse
	if(i) SDL_ShowCursor(SDL_ENABLE); else SDL_ShowCursor(SDL_DISABLE);
}

void MoveMouse(Uint16 x, Uint16 y)
{
	// Move cursor (if it's enabled) to selected position
	if(SDL_ShowCursor(SDL_QUERY) == SDL_ENABLE) SDL_WarpMouse(x, y);
}

void GetMousePosition(int *x, int *y)
{
	// Get cursor position
	if(SDL_ShowCursor(SDL_QUERY) == SDL_ENABLE) SDL_GetMouseState(x, y);
}

void DrawSurface(SDL_Surface *dst, SDL_Surface *src, int x, int y)
{
	int width = dst->w, height = dst->h; 
	int x1 = x, y1= y, x2 = 0, y2 = 0, w = src->w, h = src->h;

	// If a portion of the surface is out of the screen, only draw the portion that is visible
	if(x < 0 && (x + src->w) < width)
	{
		x1 = 0;
		x2 = -x;
		w = src->w - x2;
	}
	if(x > 0 && (x + src->w) > width)
		w = width - x;
	if(x < 0 && (x + src->w) > width)
	{
		x1 = 0;
		x2 = x * (-1);
		w = (src->w + x) - ((src->w + x) - width);
	}

	if(y < 0 && (y + src->h) < height)
	{
		y1 = 0;
		y2 = -y;
		h = src->h + y;
	}
	if(y > 0 && (y + src->h) > height)
		h = height - y;
	if(y < 0 && (y + src->h) > height)
	{
		y1 = 0;
		y2 = -y;
		h = (src->h + y) - ((src->h + y) - height);
	}

	// Blit surface
	DrawSurfacePortion(dst, src, x1, y1, x2, y2, w, h);
}

void DrawSurfaceCenter(SDL_Surface *dst, SDL_Surface *src, int x, int y)
{
	int x2, y2;
	SDL_Rect rect;

	// Calculate X and Y
	x2 = x - (src->w / 2);
	y2 = y - (src->h / 2);

	// Set area and blit surface
	rect = (SDL_Rect) {x2, y2, src->w, src->h};
	SDL_BlitSurface(src, NULL, dst, &rect);
}

void DrawSurfacePortion(SDL_Surface *dst, SDL_Surface *src, int x, int y, int x2, int y2, int w, int h)
{
	SDL_Rect rect, rect2;

	// Set area, set surface area to draw and blit surface
	rect = (SDL_Rect) {x, y, w, h};
	rect2 = (SDL_Rect) {x2, y2, w, h};
	SDL_BlitSurface(src, &rect2, dst, &rect);
}

void SaveScreenshot(SDL_Surface *screen)
{
	char c[128];
	int i, fd;

	// Check available file
	for(i = 0; i <= 999; i++)
	{
		if(i < 10) sprintf(c, "screenshot00%i.bmp", i); else
		if(i < 100) sprintf(c, "screenshot0%i.bmp", i); else
		sprintf(c, "screenshot%i.bmp", i);

		fd = open(c, O_CREAT | O_EXCL);
		if(fd) break;
	}

	// Save screenshot
	SDL_SaveBMP(screen, c);
}
